Auth

Enum Auth 

#[non_exhaustive]
pub enum Auth { Anonymous, Static(Credentials), Provider(DynCredentialsProvider), }
Authentication configuration for requests.

Most applications use one of the constructor helpers on Auth:

Optional credential features also add profile, IMDS, and STS-based constructors.

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


Anonymous

Send unsigned requests.



Static(Credentials)

Static access keys (and optional session token).



Provider(DynCredentialsProvider)

Dynamic credentials provider.

Implementations

Source

impl Auth

pub fn from_env() -> Result<Self>

Loads static credentials from standard AWS env vars.

Reads AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, and optionally AWS_SESSION_TOKEN.

pub fn provider(provider: DynCredentialsProvider) -> Self

Uses a dynamic credentials provider.

pub fn from_profile(profile: impl AsRef<str>) -> Result<Self>

Available on crate feature credentials-profile only.

Loads credentials from a named profile.

pub fn from_profile_env() -> Result<Self>

Available on crate feature credentials-profile only.

Loads credentials from the profile defined by environment variables.

pub async fn from_imds() -> Result<Self>

Available on crate features credentials-imds and async only.

Loads IMDS credentials and wraps them in a cached provider.

pub async fn from_imds_with_tls_root_store( tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-imds and async only.

Loads IMDS credentials and wraps them in a cached provider.

pub fn from_imds_blocking() -> Result<Self>

Available on crate features credentials-imds and blocking only.

Loads IMDS credentials and wraps them in a cached provider.

pub fn from_imds_blocking_with_tls_root_store( tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-imds and blocking only.

Loads IMDS credentials and wraps them in a cached provider.

pub async fn assume_role( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source_credentials: Credentials, ) -> Result<Self>

Available on crate features credentials-sts and async only.

Assumes a role using static source credentials (async).

pub async fn assume_role_with_tls_root_store( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source_credentials: Credentials, tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-sts and async only.

Assumes a role using static source credentials and a specific trust root policy (async).

pub fn assume_role_blocking( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source_credentials: Credentials, ) -> Result<Self>

Available on crate features credentials-sts and blocking only.

Assumes a role using static source credentials (blocking).

pub fn assume_role_blocking_with_tls_root_store( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source_credentials: Credentials, tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-sts and blocking only.

Assumes a role using static source credentials and a specific trust root policy (blocking).

pub async fn from_web_identity_env() -> Result<Self>

Available on crate features credentials-sts and async only.

Loads web identity credentials from env vars (async).

pub async fn from_web_identity_env_with_tls_root_store( tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-sts and async only.

Loads web identity credentials from env vars and a specific trust root policy (async).

pub fn from_web_identity_env_blocking() -> Result<Self>

Available on crate features credentials-sts and blocking only.

Loads web identity credentials from env vars (blocking).

pub fn from_web_identity_env_blocking_with_tls_root_store( tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-sts and blocking only.

Loads web identity credentials from env vars and a specific trust root policy (blocking).

pub async fn assume_role_with_provider( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source: DynCredentialsProvider, ) -> Result<Self>

Available on crate features credentials-sts and async only.

Assumes a role using a credentials provider (async).

pub async fn assume_role_with_provider_with_tls_root_store( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source: DynCredentialsProvider, tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-sts and async only.

Assumes a role using a credentials provider and a specific trust root policy (async).

pub fn assume_role_with_provider_blocking( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source: DynCredentialsProvider, ) -> Result<Self>

Available on crate features credentials-sts and blocking only.

Assumes a role using a credentials provider (blocking).

pub fn assume_role_with_provider_blocking_with_tls_root_store( region: Region, role_arn: impl Into<String>, role_session_name: impl Into<String>, source: DynCredentialsProvider, tls_root_store: CredentialsTlsRootStore, ) -> Result<Self>

Available on crate features credentials-sts and blocking only.

Assumes a role using a credentials provider and a specific trust root policy (blocking).