Simple access to Amazon Web Service’s (AWS) Simple Storage Service (S3)
Re-exports
pub use awscreds as creds;pub use awsregion as region;pub use bucket::Bucket;pub use bucket::Tag;pub use bucket_ops::BucketConfiguration;Modules
Implementation of AWS V4 Signing
Enums
AWS S3 region identifier,
passing in custom values is also possible, in that case it is up to you to pass a valid endpoint,
otherwise boom will happen :)