[−]Enum s3::Region
AWS S3 region identifier, passing in custom values is also possible, in that case it is up to you to pass a valid endpoint, otherwise boom will happen :)
Example
use std::str::FromStr; use awsregion::Region; // Parse from a string let region: Region = "us-east-1".parse().unwrap(); // Choose region directly let region = Region::EuWest2; // Custom region requires valid region name and endpoint let region_name = "nl-ams".to_string(); let endpoint = "https://s3.nl-ams.scw.cloud".to_string(); let region = Region::Custom { region: region_name, endpoint };
Variants
us-east-1
us-east-2
us-west-1
us-west-2
ca-central-1
ap-south-1
ap-northeast-1
ap-northeast-2
ap-northeast-3
ap-southeast-1
ap-southeast-2
cn-north-1
cn-northwest-1
eu-north-1
eu-central-1
eu-west-1
eu-west-2
eu-west-3
me-south-1
sa-east-1
Digital Ocean nyc3
Digital Ocean ams3
Digital Ocean sgp1
Yandex Object Storage
Wasabi us-east-1
Wasabi us-east-2
Wasabi us-west-1
Wasabi eu-central-1
Custom region
Implementations
impl Region