[−][src]Enum s3::region::Region
AWS S3 region identifier, passing in custom values is also possible, in that case it is up to you to pass a valid endpoint, otherwise boom will happen :)
Example
use std::str::FromStr; use s3::region::Region; // Parse from a string let region: Region = "us-east-1".parse().unwrap(); // Choose region directly let region = Region::EuWest2; // Custom region requires valid region name and endpoint let region_name = "nl-ams".to_string(); let endpoint = "https://s3.nl-ams.scw.cloud".to_string(); let region = Region::Custom { region: region_name, endpoint };
Variants
UsEast1us-east-1
UsEast2us-east-2
UsWest1us-west-1
UsWest2us-west-2
CaCentral1ca-central-1
ApSouth1ap-south-1
ApNortheast1ap-northeast-1
ApNortheast2ap-northeast-2
ApNortheast3ap-northeast-3
ApSoutheast1ap-southeast-1
ApSoutheast2ap-southeast-2
CnNorth1cn-north-1
CnNorthwest1cn-northwest-1
EuCentral1eu-central-1
EuWest1eu-west-1
EuWest2eu-west-2
EuWest3eu-west-3
SaEast1sa-east-1
DoNyc3Digital Ocean nyc3
DoAms3Digital Ocean ams3
DoSgp1Digital Ocean sgp1
CustomCustom region
Fields of Custom
Methods
impl Region[src]
pub fn endpoint(&self) -> String[src]
pub fn scheme(&self) -> String[src]
pub fn host(&self) -> String[src]