Implementation of AWS V4 Signing
Constants
Functions
- authorization_
header - Generate the AWS authorization header.
- authorization_
query_ params_ no_ sig - canonical_
header_ string - Generate a canonical header string from the provided headers.
- canonical_
query_ string - Generate a canonical query string from the query pairs in the given URL.
- canonical_
request - Generate a canonical request.
- canonical_
uri_ string - Generate a canonical URI string from the given URL.
- flatten_
queries - scope_
string - Generate an AWS scope string.
- signed_
header_ string - Generate a signed header string from the provided headers.
- signing_
key - Generate the AWS signing key, derived from the secret key, date, region, and service name.
- string_
to_ sign - Generate the “string to sign” - the value to which the HMAC signing is applied to sign requests.
- uri_
encode - Encode a URI following the specific requirements of the AWS service.